"""
This module defines default entities and actions for CiviCRM APIv4.
"""

ENTITIES = [
    'ACL',
    'ACLEntityRole',
    'ActionSchedule',
    'Activity',
    'ActivityContact',
    'Address',
    'Batch',
    'Contact',
    'ContactType',
    'Contribution',
    'ContributionPage',
    'ContributionProduct',
    'ContributionRecur',
    'ContributionSoft',
    'Country',
    'County',
    'CustomField',
    'CustomGroup',
    'Dashboard',
    'DashboardContact',
    'DedupeException',
    'DedupeRule',
    'DedupeRuleGroup',
    'Domain',
    'Email',
    'Entity',
    'EntityBatch',
    'EntityFinancialAccount',
    'EntityFinancialTrxn',
    'EntityTag',
    'Event',
    'ExampleData',
    'Extension',
    'File',
    'FinancialAccount',
    'FinancialItem',
    'FinancialTrxn',
    'FinancialType',
    'Group',
    'GroupContact',
    'GroupNesting',
    'GroupOrganization',
    'IM',
    'Job',
    'JobLog',
    'LineItem',
    'LocBlock',
    'LocationType',
    'Log',
    'MailSettings',
    'Mailing',
    'MailingEventBounce',
    'MailingEventConfirm',
    'MailingEventOpened',
    'MailingEventQueue',
    'MailingEventSubscribe',
    'MailingEventUnsubscribe',
    'MailingGroup',
    'MailingJob',
    'Managed',
    'Mapping',
    'MappingField',
    'Membership',
    'MembershipBlock',
    'MembershipStatus',
    'MembershipType',
    'MessageTemplate',
    'Navigation',
    'Note',
    'OpenID',
    'OptionGroup',
    'OptionValue',
    'PCP',
    'PCPBlock',
    'Participant',
    'PaymentProcessor',
    'PaymentProcessorType',
    'PaymentToken',
    'Permission',
    'Phone',
    'PriceField',
    'PriceFieldValue',
    'PriceSet',
    'PriceSetEntity',
    'PrintLabel',
    'Product',
    'Queue',
    'RecentItem',
    'Relationship',
    'RelationshipCache',
    'RelationshipType',
    'ReportInstance',
    'Route',
    'SavedSearch',
    'SearchDisplay',
    'SearchSegment',
    'Setting',
    'StateProvince',
    'StatusPreference',
    'SubscriptionHistory',
    'System',
    'Tag',
    'Translation',
    'UFField',
    'UFGroup',
    'UFJoin',
    'UFMatch',
    'UserJob',
    'Website',
    'WordReplacement',
    'WorkflowMessage',
    'WorldRegion',
]
"""
All standard entities for CiviCRM APIv4.
"""

ACTIONS = [
    'checkAccess',
    'getActions',
    'getFields',
    'get',
    'create',
    'update',
    'save',
    'delete',
    'replace',
]
"""
All standard actions for CiviCRM APIv4.
"""
